local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local BS = AceLibrary("Babble-Spell-2.2")
local RL = AceLibrary("RosterLib-2.0")

GridStatusMissingBuffs = GridStatus:NewModule("GridStatusMissingBuffs")
GridStatusMissingBuffs.menuName = "Missing Buffs"

local function statusForSpell(spell)
	local prefix = "buff_"
	return prefix .. string.gsub(spell, "[^%a]", "")
end

GridStatusMissingBuffs.defaultDB = {
	debug = false, 
	buff_MarkoftheWild = {
		text = "Mark of the Wild",
		desc = "Buff Group: Mark of the Wild",
		icon = "buff_MarkoftheWild",
		buffs = {
			buff_MarkoftheWild = "Mark of the Wild",
			buff_GiftoftheWild = "Gift of the Wild",
		},
		enable = true,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buff_Fortitude = {
		text = "Fortitude",
		desc = "Buff Group: Fortitude",
		icon = "buff_PrayerofForitude",
		buffs = {
			buff_PowerWordForitude = "Power Word: Fortitude",
			buff_PrayerofForitude = "Prayer of Fortutde",
		},
		enable = true,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buff_DivineSpirit = {
		text = "Divine Spirit",
		desc = "Buff Group: Divine Spirit",
		icon = "buff_DivineSpirit",
		buffs = {
			buff_DivineSpirit = "Divine Spirit",
			buff_PrayerofSpirit = "Prayer of Spirit",
		},
		enable = true,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buff_ArcaneIntellect = {
		text = "Arcane Intellect",
		desc = "Buff Group: Arcane Intellect",
		icon = "buff_ArcaneBrilliance",
		buffs = {
			buff_ArcaneIntellect = "Arcane Intellect",
			buff_ArcaneBrilliance = "Arcane Brilliance",
		},
		enable = true,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
}
 
function GridStatusMissingBuffs:OnInitialize()
	self.super.OnInitialize(self)

	self:RegisterStatuses()
end

function GridStatusMissingBuffs:OnEnable()
	self.debugging = self.db.profile.debug
	self:CreateAddRemoveOptions()
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("SpecialEvents_UnitBuffGained", "UpdateUnit")
	self:RegisterEvent("SpecialEvents_UnitBuffLost", "UpdateUnit")
	self:RegisterEvent("Grid_UnitJoined")
	self:CreateAddRemoveOptions()
end

function GridStatusMissingBuffs:Grid_UnitJoined(name, unitid)
	self:UpdateUnit(unitid)
end

function GridStatusMissingBuffs:UpdateUnit(unitid)
	local status, settings

	for status, settings in self:ConfiguredStatusIterator() do
		self:ShowMissingBuffs(unitid, status, settings)
	end
end

function GridStatusMissingBuffs:RegisterStatuses()
	local status, settings, desc

	for status, settings in self:ConfiguredStatusIterator() do
		desc = settings.desc or settings.text
		self:Debug("registering", status, desc)
		self:RegisterStatus(status, desc, self:GetBuffGroupOptions(status))
	end
end

function GridStatusMissingBuffs:UnregisterStatuses()
	local status, moduleName, desc
	for status, moduleName, desc in self.core:RegisteredStatusIterator() do
		if moduleName == self.name then
			self:UnregisterStatus(status)
			self.options.args[status] = nil
		end
	end
end

function GridStatusMissingBuffs:ConfiguredStatusIterator()
	local profile = self.db.profile
	local status

	return function ()
		status = next(profile, status)

		-- skip any non-table entries
		while status ~= nil and type(profile[status]) ~= "table" do
			status = next(profile, status)
		end

		if status == nil then
			return nil
		end

		return status, profile[status]
	end
end

function GridStatusMissingBuffs:GetBuffGroupOptions(status)
	return {
		["add_buff"] = {
			type = "text",
			name = "Add Buff",
			desc = "Adds a new buff to the status module",
			get = false,
			usage = "<buff name>",
			set = function(v) self:AddBuff(status, v) end,
			order = 201
		},
		["delete_buff"] = {
			type = "group",
			name = "Delete Buff",
			desc = "Deletes an existing buff from the status module",
			disable = function() return next(self.db.profile[status].buffs) == nil end,
			args = {},
			order = 202
		},
		["missing_icon"] = {
			type = 'text',
			name = "Missing Icon",
			desc = "Set the Icon to appear when no buffs in the buff group are active.",
			usage = "<setName>",
			validate = self.db.profile[status].buffs,
			disable = function() return next(self.db.profile[status].buffs) == nil end,
			get = function()
				return self.db.profile[status].icon
			end,
			set = function(v)
				self.db.profile[status].icon = v
			end,
		}
	}
end

function GridStatusMissingBuffs:Reset()
	self.super.Reset(self)

	self:UnregisterStatuses()
	self:RegisterStatuses()
	self:CreateAddRemoveOptions()
end

function GridStatusMissingBuffs:CreateAddRemoveOptions()
	local status, settings

	self.options.args["add_buff_group"] = {
		type = "text",
		name = "Add Buff Group",
		desc = "Adds a new buff group to the status module",
		get = false,
		usage = "<buff name>",
		set = function(v) self:AddBuffGroup(v) end,
		order = 201
	}
	self.options.args["delete_buff_group"] = {
		type = "group",
		name = "Delete Buff Group",
		desc = "Deletes an existing buff group from the status module",
		disable = function() return next(self.db.profile[status]) == nil end,
		args = {},
		order = 202
	}

	for status, settings in self:ConfiguredStatusIterator() do
		local buffName = (settings.desc or settings.text)
		self.options.args["delete_buff_group"].args[status] = {
			type = "execute",
			name = buffName,
			desc = string.format("Remove %s from the menu", buffName),
			func = function() return self:DeleteBuffGroup(status) end
		}

		local buffId, buffName
		for buffId, buffName in pairs(settings.buffs) do
			local buffId, buffName = buffId, buffName
			self.options.args[status].args["delete_buff"].args[buffId] = {
				type = "execute",
				name = buffName,
				desc = string.format("Remove %s from the menu", buffName),
				func = function() return self:DeleteBuff(status, buffId) end
			}
		end
	end
end

function GridStatusMissingBuffs:AddBuffGroup(name)
	local status = statusForSpell(name)
	local desc = "Buff Group: "..name

	self.db.profile[status] = {
		text = name,
		desc = desc,
		enable = true,
		buffs = {
			[status] = name,
		},
		icon = status,
		priority = 90,
		range = true,
		color = { r = .5, g = .5, b = .5, a = 1 },
	}

	self:RegisterStatus(status, desc, self:GetBuffGroupOptions(status))
	self:CreateAddRemoveOptions()
end

function GridStatusMissingBuffs:DeleteBuffGroup(status)
	self:UnregisterStatus(status)
	self.options.args[status] = nil
	self.options.args["delete_buff_group"].args[status] = nil
	self.db.profile[status] = nil
end

function GridStatusMissingBuffs:AddBuff(status, name)
	local buffId = statusForSpell(name)

	self.db.profile[status].buffs[buffId] = name
	self:CreateAddRemoveOptions()
end

function GridStatusMissingBuffs:DeleteBuff(status, buffId)
	self.options.args[status].args["delete_buff"].args[buffId] = nil
	self.db.profile[status].buffs[buffId] = nil
end

function GridStatusMissingBuffs:PLAYER_REGEN_ENABLED()
	self:RegisterEvent("SpecialEvents_UnitBuffGained", "UpdateUnit")
	self:RegisterEvent("SpecialEvents_UnitBuffLost", "UpdateUnit")
	for u in RL:IterateRoster() do
		self:UpdateUnit(u.unitid)
	end
end

function GridStatusMissingBuffs:PLAYER_REGEN_DISABLED()
	self:UnregisterEvent("SpecialEvents_UnitBuffGained")
	self:UnregisterEvent("SpecialEvents_UnitBuffLost")
	for status in pairs(self.db.profile) do
		for name, status in self.core:CachedStatusIterator(status) do
			self.core:SendStatusLost(name, status)
		end
	end
end

function GridStatusMissingBuffs:SpecialEvents_UnitBuffGained(unit)
	self:UpdateUnit(unit)
end

function GridStatusMissingBuffs:SpecialEvents_UnitBuffLost(unit)
	self:UpdateUnit(unit)
end

function GridStatusMissingBuffs:ShowMissingBuffs(unit, status, settings)
	for buffId, buff in pairs(settings.buffs) do
		if Aura:UnitHasBuff(unit, buff) then
			self.core:SendStatusLost(UnitName(unit), status)
			return
		end
	end
	if self.core:GetCachedStatus(UnitName(unit), status) then
		return
	end
	local tex
	if settings.icon then
		tex = BS:GetSpellIcon(settings.buffs[settings.icon])
	end
	self.core:SendStatusGained(
		UnitName(unit),
		status,
		settings.priority,
		(settings.range and 40),
		settings.color,
		settings.text,
		nil,
		nil,
		tex)
end
